@@ -11,6 +11,13 @@ impl Utils {
         }
     }
 
+    /// To be used by sink stages to track progress
+    pub fn track_source_progress(&self, event: &Event) {
+        if let Some(metrics) = &self.metrics {
+            metrics.on_source_event(event);
+        }
+    }
+
     /// To be used by sink stages to track progress
     pub fn track_sink_progress(&self, event: &Event) {
         let point = match (event.context.slot, &event.context.block_hash) {
@@ -22,6 +29,14 @@ impl Utils {
             cursor.set_cursor(point).ok_or_warn("failed to set cursor")
         }
 
-        // TODO: add here future telemetry implementation
+        if let Some(metrics) = &self.metrics {
+            metrics.on_sink_event(event);
+        }
+    }
+
+    pub fn track_chain_tip(&self, tip: u64) {
+        if let Some(metrics) = &self.metrics {
+            metrics.on_chain_tip(tip);
+        }
     }
 }
